# Licensed Materials - Property of IBM
# 
# 09N6996 
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

sm_log () {

    # Only log if the log directory exists
    if [ -d $LOG_DIR ]; then

        if [[ "$LOG_VERBOSE" == "verbose" ]]; then
        	
        	local trc_data=
        	if [ -n "$SMHA_LOG_ID" ]; then
        	   trc_data="${SMHA_LOG_ID}:${PROGNAME}"
        	else
        	   trc_data="$PROGNAME"
            fi
            echo -e `date +%F\ %T` "$trc_data: $*" >>$LOG
        fi
        
    fi
}

gen_log_id () {
	if [ -z "$SMHA_LOG_ID" ]; then
		local nano=`date +%N`
	    SMHA_LOG_ID=${nano:3:4}
	fi
}

sm_msg () {
	
	local progname	
	if [ -n "$EXTPROGNAME" ]; then
	    progname=$EXTPROGNAME
	else
	    progname=$PROGNAME
	fi
	# Put it in the system log
	logger -t "$progname" "$*"
	# Put it on the our log
    sm_log "$*"	
    # Put it to stdout
    echo -e "$progname: $*"	
}

fatal_error_rc () {
    # The first parameter must be the exit code
    local RC=$1
    shift
    
    local progname  
    if [ -n "$EXTPROGNAME" ]; then
        progname=$EXTPROGNAME
    else
        progname=$PROGNAME
    fi

    
    if [[ "$LOGCOMMAND" == "ON" ]]; then         
        log_command_failed
        # Put it to stdout
        echo -e "$progname: $*" 1>&2
    else
        # Put in the system log and stderr
        logger -s -t "$progname" "$*"
    fi
    
    sm_log "$*"  
    
    exit $RC
}

fatal_error () {
    fatal_error_rc 1 $*    
}


return_error () {
    # The first parameter must be the exit code    
    local RC=$1
    shift
    
    # Put in the system log and stderr
    logger -s -t "$PROGNAME" "$*"
    sm_log "$*"  #logger above will send to standard error
    exit $RC
}


sm_sys_log () {	
	logger -i -t "$PROGNAME" ${1}
}


debug_internal () {

	# Only print to the screen if debug is on
    if [[ "$DEBUG" == "ON" ]]; then
        echo "$*"
    fi
    # Always place debug in the log
    sm_log "$*"
}

debug () {

	# Check if we only want to debug one program	
	if [[ -n "$DEBUG_PGM_ONLY" ]]; then
		# Only debug if this is the program to debug
		if [[ "$PROGNAME" == "$DEBUG_PGM_ONLY" ]]; then
			debug_internal "$*"
		fi
	else
		debug_internal "$*"
	fi
}

debug_tsa () {
	# TSA scripts cause a lot of debug to the log. Only debug if if DEBUG_TSA is on.
    if [[ "$DEBUG_TSA" == "ON" ]]; then
        debug "TSA: $*"
    fi
    
}

record_tsa_action () {
    local action=$1
    local name=$2
    local rc=$3
    shift 3

    debug "TSA_ACTION:$action:$name:$rc:"
}

debug_if_not_null () {
    if [ -n "$*" ]; then
    	debug $*   
    fi	
}

mk_log_dir () {
    if [ ! -d $LOG_DIR ]; then
        mkdir -p $LOG_DIR
    fi  
}

# Returns 1 if this option should not be saved
# Parameters
#   option to check
# Relies on
#   PARM_DONT_SAVE
never_save_opt () {
	local i
	# if there is a list of options not to save
	if [ -n "$PARM_DONT_SAVE" ]; then	
	   for i in ${PARM_DONT_SAVE}
	   do
		  if [[ "$i" == "$1" ]]; then
			 return 1
		  fi
	   done
    fi
	return 0
	
}


# Append the set of variables to the cfg file
# Parameters are:
#   variable names
# Return value:
#  
append_to_cfg_file () {
	local i
	
	for i in $*
	do
		# convert the variable name back to an option name
		local option_name=${i//_/-}
		
		# if this is a multiple parm, then delete the __1 part
		option_name=${option_name%%--*}
		
		# Don't save this option if we shouldn't
		never_save_opt $option_name
		if [ $? -ne 1 ]; then
    		# write the option and value
	       	echo "--${option_name}=${!i}" >> $SMHA_CFG_FILE
		fi
	done
	return 0
}

# Append the string to the cfg file
# Parameters are:
#   string to append
# Return value:
#  
append_string_to_cfg_file () {
    local i
    
    for i in $*
    do
        echo "${i}" >> $SMHA_CFG_FILE
    done
    return 0
}


# Remove the cfg file
# Parameters are:
#   none
# Return value:
#  
remove_cfg_file () {
    if [ -f $SMHA_CFG_FILE ]; then
        rm $SMHA_CFG_FILE
    fi
}

# Create a new cfg file
# Parameters are:
#   variable names
# Return value:
#  
create_cfg_file () {
    remove_cfg_file
    append_to_cfg_file $*
}

# Get a parameter value from the config file
# Parameters
#   Parameter to retrieve
# Relies on 
#   SMHA_CFG_FILE
# Returns
#   Sets the global variable CFG_RETURN_VALUE
#
get_value_from_cfg () {
    CFG_RETURN_VALUE=
    
    # Make sure the conf file exists
    if [ ! -f $SMHA_CFG_FILE ]; then
    	return 1
    fi
    
    #Set the search value
    local srch_option=\\-\\-${1}=
	local option_line=`grep "$srch_option" $SMHA_CFG_FILE`
#	debug "srch_option: $srch_option"
#	debug "option_line: $option_line"
	if [ -n "$option_line" ]; then
        # Get the value: delete everything up to the first = from the left
        CFG_RETURN_VALUE=${option_line#*=}
    else
        return 1            
	fi
	return 0
}

remove_option_from_cfg_file () {
	local opt=${1}
	
    # convert the variable name back to an option name
    local option_name=${opt//_/-}
    
    # if this is a multiple parm, then delete the __1 part
    option_name=${option_name%%--*}

    local sed_s="s/--$option_name.*//;"

    # Create a temp file
    local tmpf=`mktemp /tmp/smha_rofcf.XXXXXX`  || fatal_error "`eval_gettext "Could not make a temporary file."`"
    # Copy the contents to it
    cp $SMHA_CFG_FILE $tmpf    
    # Make the updates and write to the original
    debug "Removing $opt from config file"
    sed "$sed_scr" <$tmpf  >$SMHA_CFG_FILE        
    rm $tmpf                    
	
}

remove_from_cfg_file () {
	local opt
	
	for opt in $*
	do
	   remove_option_from_cfg_file $opt
	done
	
}

does_ha_config_exist () {
# Just check for a config file

    if [ -e "$SMHA_CFG_FILE" ]; then
        return 1
    else
        return 0
    fi

}

# Check if the TSA domain is already setup
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   1 if the domain is setup
is_domain_setup () {	

	lsrpdomain -x $TSA_DOMAIN &> /dev/null	
	if [ $? -eq 0 ]; then
		return 1
	else
	    return 0
    fi
}

# Wait for the domain to come online
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain is online, non-zero if the domain timed out or an error
wait_for_domain () {
	
	local i
	for i in {1..160}
    do
        local status=`LC_ALL=C lsrpdomain -x -l $TSA_DOMAIN 2> /dev/null | grep "OpState" | awk '{print $3}'`
        debug "wait_for_domain: status: $status"
        if [[ "$status" == "Online" ]]; then
            return 0
        else
            sleep 2
            # Stop this from looping if the domain doesn't exist.
            lsrpdomain -x -l $TSA_DOMAIN &> /dev/null
            if [ $? -eq 6 ]; then
            	fatal_error "`eval_gettext "The domain no longer exists."`"
            fi
            sleep 2
        fi
    done
    return 1
}

# Start and wait for the domain to come online
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain is online, non-zero if the domain timed out or an error
start_n_wait_for_domain () {

    local result=`startrpdomain $TSA_DOMAIN 2>&1`
    if [ $? -ne 0 ]; then
    	sm_msg "`eval_gettext \"Error starting the domain: \\\$result\"`"
    fi
    debug "startrpdoman result:$result"
    
	sleep 10                
    wait_for_domain
    if [ $? -ne 0 ]; then
        fatal_error "`eval_gettext "Timed out waiting for TSA domain to come on line."`"
    fi    
}

# Remove and wait for the domain to be deleted
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain was deleted, non-zero if the remove timed out or an error
remove_n_wait_for_domain () {
	
    debug "Attempt to remove domain"
    local result=`rmrpdomain $TSA_DOMAIN 2>&1`
    if [ $? -ne 0 ]; then
    	sm_msg "`eval_gettext \"Error removing the domain: \\\$result\"`"
    fi
    debug "$result"
                
    local i
    local rmdr
    for i in {1..160}
    do
    	debug "See if the domain has been removed."
        lsrpdomain -x $TSA_DOMAIN &> /dev/null
        local rc=$?  
        debug "lsrpdomain: rc: $rc"
        if [ $rc -eq 6 ]; then
            return 0
        else
            # It's still there, try to remove it again every so often
            let "rmdr = i % 5"
            if [ $rmdr -eq 0 ]; then
                # Make sure all the rg's are set to offline
                take_all_rg_offline            
                # Try deleting all the resources
                rm_all_rsrc
                sleep 2
            	rmrpdomain $TSA_DOMAIN
            fi
            
            sleep 5
        fi
    done

    return 1    
}

#is_rg_created () {
#	local rg=${1}
#	local found_rg=`lsrg | grep $rg | cut -d' ' -f1`
#	
#	debug "found rg:$found_rg:$rg"
#	if [ x"$found_rg" == x"$rg" ]; then
#		debug "is_rg_created: $rg =Y"
#		return 1 
#	fi
#	
#	debug "is_rg_created: $rg =N"
#	return 0
#}

is_rg_created () {
    local rg=${1}
    
    local i
    for i in {1..10}
    do
    	    
	    lsrg -g $rg &>/dev/null
	    local lsrg_rc=$?
	    
	    debug "found rg:$rg:$lsrg_rc"
	    case $lsrg_rc in
	    6 )
	        return 0
	    ;;
	    0 )
	        return 1
	    ;;
	    *)
	        sleep 5
	    ;;
	    esac    
    done
}



# Get the TSA rg status
# Parameters are:
#   rg name
# Return value:
#   the string value from TSA, like "online","offline", etc
get_rg_status ()
{    
    local rg=${1}

    rsrcstatus=`LC_ALL=C lsrg -A d -g $rg | grep "OpState" | awk '{print $3}'`

    GET_RSRC_STATUS=$rsrcstatus
    debug "rg status of $rg:$rsrcstatus"
    return 0       
}

# Get the TSA rg status
# Parameters are:
#   rg name
# Return value:
#   the OpState numeric value from TSA, like 0-6
# 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline
get_rg_status_num ()
{    
    local rg=${1}

    rg_status=`LC_ALL=C lsrg -A d -g $rg OpState | grep "OpState" | cut -d' ' -f3`

    debug "rg status of $rg:$rg_status"
    return $rg_status       
}

    
wait_for_rg_offline () {
    local rg=${1}
    local i=
    
    is_rg_created $rg
    if [ $? -ne 1 ]; then
    	return 0
    fi
    
    for i in {1..160}
    do
        debug "See if $rg is offline"        
        get_rg_status_num $rg
        case $? in
        2|3 )
            return 0
        ;;
        4 )
            # stuck online, just fail
            return 1
        ;;
        *)
            sleep 5
        ;;
        esac
    done
    return 1
}


    
wait_for_mq_online () {
  
    local i=
    
    for i in {1..160}
    do
        debug "See if mq is online"        
        /etc/rc.d/smhamq status
        case $? in
        0 )
            return 0
        ;;
        *)
            # not online yet
            sleep 1
        ;;
        esac
    done
    return 1
}

wait_for_daemon_online () {
  
    local i=
    
    for i in {1..160}
    do
        debug "See if daemon is online"        
        /etc/rc.d/smhadm status
        case $? in
        0 )
            return 0
        ;;
        *)
            # not online yet
            sleep 1
        ;;
        esac
    done
    return 1
}
    
# Parameters are:
#   resource group name
take_rg_offline () {
    local rg=${1}
    
    is_rg_created $rg
    if [ $? -eq 1 ]; then
        #chrg -o offline ${rg}    
        debug "tsa call hook 2"
        call_tsa_chrg_o_l_u -o offline $rg
    fi
    
}

# Parameters are:
#   resource group name
take_rg_offline_n_wait () {
    local rg=${1}
    
    take_rg_offline ${rg}
    wait_for_rg_offline ${rg}
    if [ $? -ne 0 ]; then
        fatal_error "`eval_gettext \"Could not take resource group \\\$rg offline.\"`"
    fi
    return 0
}

take_rgs_offline () {
	local rgs=$*
	
	local rg=
	for rg in $rgs
	do
		take_rg_offline_n_wait_nonfatal $rg
	done
}

# Parameters are:
#   resource group name
take_rg_offline_n_wait_nonfatal () {
    local rg=${1}
    
    take_rg_offline ${rg}
    wait_for_rg_offline ${rg}
   
    return $?
    
}

take_all_rg_offline () {
	

    local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rg
    for rg in $rglist
    do
        debug "Take $rg offline"
        take_rg_offline $rg        
    done    
	
}

rm_all_rg () {
    

    local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rg
    for rg in $rglist
    do
        debug "Remove $rg"
        rmrg $rg        
    done    
    
}


rm_all_rsrc () {
    local cmd_out=`LC_ALL=C rmrsrc -s "Name like '%'" IBM.Application Force=1 2>&1`
    debug "rm_all_rsrc:$cmd_out"
}

wait_all_rg_offline() {
	
	local we_done
    local statelist
    local state
    local i
        
    for i in {1..20}
    do   
        we_done=Y
        local statelist=`LC_ALL=C lsrg OpState | grep "OpState" | awk '{print $3}'`
	    for state in $statelist
	    do
	        debug "Take rg state is (${TSA_OFFLINE}=offline):$state"
	        case $state in 
	        	$TSA_OFFLINE | $TSA_FAILED_OFFLINE)
	        	    # Nothing to do... don't set the status - we_done=Y
	        	    echo " " > /dev/null
        	    ;;
	        	$TSA_UNKNOWN)
                    # If the state is unknown after a few iterations count it as offline.
	                if [ $i -le 2 ]; then
                        we_done=N
                    fi
        	    ;;    	
	        	*)
                    we_done=N
	        	;;
	        esac	        
        
	    done
	    
	    # If we didn't find any that weren't offline, the return
	    if [[ "$we_done" == "Y" ]]; then
	    	return
	    fi
	    # If this is the last loop then we timed out
        if [ $i -eq 20 ]; then
        	return 1
        fi
	        
	    sleep 10
    done
}

wait_for_rg_online () {
    local rg=${1}
    local i=
    for i in {1..160}
    do
        debug "See if $rg is online"        
        get_rg_status_num $rg
        case $? in
        1)
            return 0
        ;;
        *)
            sleep 5
        ;;
        esac
        
    done
    return 1
}

start_n_wait_for_rg_online () {
    local rg=${1}
    
    ch_rg_online ${1}   
    sleep 2
	wait_for_rg_online $rg
}

ch_rg_online () {
    local rg=${1}
    local rc
    
    # There are timing issues that can occur, so try changing the status and look for errors
    local i
    local cmd_out
    for i in {1..15}
    do
        debug "Change the rg $rg to online"        
        #local cmd_out=`chrg -o online $rg 2>&1`
        debug "tsa call hook 3"
        cmd_out=$(call_tsa_chrg_o_l_u -o online $rg)
        rc=$?
        case $rc in
        0)
            return 0
        ;;
        *)
            sleep 7
        ;;
        esac
        
    done
    debug "chrg output: $cmd_out"
    debug "Could not change nominal state of $rg to online. rc=$rc"
    return $rc
}

start_all_rg_online () {
    
    local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rg
    for rg in $rglist
    do
        debug "Set nominal state of $rg online"
        ch_rg_online $rg        
    done    
    
}


set_dir_ha_props () {
# Settings configured via <director>/data/TWGServer.prop
# HA.enabled = true|false
# HA.store.backup = true|false
# HA.events.shutdowndelay = 10

    local twgfile=$INSTALL_DIR/data/TWGServer.prop

	isset=`grep "HA.enabled" $twgfile | grep "true"`
	if [ -z "$isset"  ]; then
	    echo "HA.enabled = true" >>$twgfile
#	    echo "HA.store.backup = true" >>$twgfile
#	    echo "HA.events.shutdowndelay = $DIR_SHUTDOWN_DELAY" >>$twgfile
	fi

}

remove_dir_ha_props () {

    local twgfile=$INSTALL_DIR/data/TWGServer.prop

    perl -pi -e "s/^HA.enabled.*=.*//g" $twgfile
    
}


# Parse a network alias string into pieces
# Parameters are:
#   network alias
# Return value:
#   Parsed pieces via the variables:
#     ALIAS_ADDR
#     ALIAS_MASK
#     ALIAS_NIC
parse_network_alias () {
	# Example: --network-alias=9.5.1.4,255.255.255.0,eth0
	#                          = Address,NetMask,NIC
	local float=${1}
    local float_sep=`echo $float | sed 's/,/ /g' `

	ALIAS_ADDR=`echo $float_sep | awk '{print $1}'`
	if [ -z "$ALIAS_ADDR" ]; then
	    debug "ALIAS_ADDR not set. $float" 
		fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.\"`"
	fi
	ALIAS_MASK=`echo $float_sep | awk '{print $2}'`
    if [ -z "$ALIAS_MASK" ]; then
        debug "ALIAS_MASK not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.\"`"
    fi
	ALIAS_NIC=`echo $float_sep | awk '{print $3}'`
    if [ -z "$ALIAS_NIC" ]; then
        debug "ALIAS_NIC not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.\"`"
    fi			
}

# Parse a network alias6 string into pieces
# Parameters are:
#   network alias6
# Return value:
#   Parsed pieces via the variables:
#     ALIAS_ADDR
#     ALIAS_PREFIX (Optional)
#     ALIAS_NIC
parse_network_alias6 () {
	# Example: --network-alias6=2001:db8::1428:57ab,64,eth0
	#                          = Address,NetPrefix,NIC
		
	local float=${1}
	
    ALIAS_NIC=${float##*,}
    local TEMP=${float#*,}
    ALIAS_PREFIX=${TEMP%%,*}
    ALIAS_ADDR=${float%%,*}
	
	if [ -z "$ALIAS_ADDR" ]; then
	    debug "ALIAS_ADDR not set. $float" 
		fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.\"`"
	fi
    if [ -z "$ALIAS_NIC" ]; then
        debug "ALIAS_NIC not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.\"`"
    fi			
}


# Returns 1=true, 0=false
is_mounted () {
	local mp=${1}    
    local foundmp=`grep " $mp " /proc/mounts | cut -d' ' -f2`
    if [ "x${foundmp}" == "x$mp" ]; then
    	debug "is_mounted: $mp found mounted: $foundmp"
        return 1
    else
        #debug "$mp is not mounted: ${foundmp} vs $mp"
        return 0
    fi
}

check_for_dependent_mirror () {    
	# Intent is to check if the mirror name passed in has another mirror
	# that is flagged as being dependent upon it.
	local mirror_to_check=${1}
    # Get the list of mirrors        
    tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        # check if this mirror has a dependency. 
        local dependency=`$GETMIRROR_DEPENDENCY_CMD -p $mmir`
        local dependent=$?
        if [ -z $dependency ]; then
            	debug "check_for_dependent_mirror: No dependency found for mount point $mmir"
        else 
        	# the mirror $mmir has a dependency on $dependency.
        	# Is $dependency the same name as $mirror_to_check?
        	if [ "$dependency" == "$mirror_to_check" ]; then
        		debug "check_for_dependent_mirror: $mmir is dependent on $mirror_to_check"
        		return 1
        	fi
        fi
    done < $tmpf    
    rm $tmpf
	return 0
}

mk_bkup_file_if_not_already () {
    local ffile=${1}
    local bkupfile=${2}
    	
    # Make a backup copy of the file if not there already
    if [ ! -f $bkupfile ]; then
        debug "Making backup of $ffile"
        cp -a $ffile $bkupfile
    fi

}


chk_nodes_opstate () {
    
    for opstate in `LC_ALL=C lsrsrc IBM.PeerNode OpState | grep OpState | cut -d' ' -f3`
    do
        if [ "$opstate" != "1" ]; then          
            # If the operational state is not 1 
            fatal_error "`eval_gettext "Not all nodes in the cluster are online."`"
        fi          
    done
    
    
}

is_node_online () {
	
	local node=${1}
	local RC=0
    
    local opstate=`LC_ALL=C lsrsrc -s "Name='$node'" IBM.PeerNode OpState | grep OpState | cut -d' ' -f3`
    if [ "$opstate" == "1" ]; then
        RC=1    	          
    fi          
    debug "is_node_online: Node: $node Online: $RC"
    return $RC
    
}

is_director_online () {
    local RC=0
    
    local opstate=`LC_ALL=C lsrsrc -s "Name='$DIRECTOR_RSRC_NAME' && ResourceType=1" IBM.Application OpState | grep OpState | cut -d' ' -f3`
    if [ "$opstate" == "1" ]; then
        RC=1                  
    fi          
    debug "is_director_online: Online: $RC"
    return $RC
}




get_rsct_node_id () {
    local nodename=${1}

    local line=`grep $nodename $RSCT_NODEINFO_FILE`
    NODEID=${line##*0x}

}

set_rsct_node_id () {
    local nodeid=${1}
    
    if [ -e "$RSCT_NODEID_FILE" ]; then
        rm $RSCT_NODEID_FILE
    fi
    echo $nodeid >$RSCT_NODEID_FILE
    echo " " >>$RSCT_NODEID_FILE
    echo "# The first line of this file contains the RSCT node id of this" >>$RSCT_NODEID_FILE
    echo "# machine.  Please do not delete or modify it." >>$RSCT_NODEID_FILE
    
     /usr/sbin/rsct/install/bin/recfgct -s
    
}


is_address_ipv6 () {
	local address=${1}
	local ipv6=`echo $address | grep :`
	
	if [ -n "$ipv6" ]; then
	   return 1
	else
	   return 0
    fi
}

record_starting_termination () {
    local terminationFile=$SMHA_CONFIGDIR/terminating
    touch $terminationFile	
}

is_terminating () {
    local terminationFile=$SMHA_CONFIGDIR/terminating
    
    if [ -e "$terminationFile" ]; then    	
       return 1
    else
       return 0
    fi
	
}

is_configured () {
    local RC=0  
    if [ -f $SMHA_CFG_FILE ]; then
        # It's there but is the node parameter present?
        get_value_from_cfg node
        if [ $? -eq 0 ]; then
        	# And the domain seems to be setup?
        	is_domain_setup
            if [ $? -eq 1 ]; then       
                RC=1
            fi
        fi
    fi

    return $RC            
}


retry_cmd () {
    local times=$1
    local expect_rc=$2
    shift 2
    local run_cmd="$*"

    debug "retry_cmd: times:$times rc:$expect_rc Command: $run_cmd"         
   
    local i
    for i in $(seq 1 ${times})
    do 
        debug "Retry Command: $run_cmd"         
        local cmd_out=`$run_cmd 2>&1`
        rc=$?
        debug "Retry: expect_rc=$expect_rc, rc=$rc"
        debug "Retry cmd_out:$cmd_out"
        if [[ "$rc" == "$expect_rc" ]]; then
            return $rc
        else
           debug "retry_cmd:$run_cmd"
           debug "output:$cmd_out" 
           sleep 2     
        fi
    done
    
}

make_relationship () {
    local relation=${1}
    local source=${2}
    local target=${3}
    local name=${4}
   
    local i 
    for i in {1..60}
    do
    
        debug "mkrel -p $relation -S $source -G ${target} ${name}"
        local cmd_out=`LC_ALL=C mkrel -p $relation -S $source -G ${target} ${name} 2>&1`
        
        relationship_exists ${name}
        if [ $? -eq 1 ]; then
            debug "Made relationship $name"
            
            return 0
        else
            debug "mkrel cmd_out: $cmd_out"
            debug "Retry making relationship $name"
            sleep 5       
        fi
    done
    return 1
}

relationship_exists () {
    local name=${1}
    
    present=`LC_ALL=C lsrel | awk '{print $1}' | grep ${name}`
    if [[ "$present" == "$name" ]]; then
        return 1
    else
        return 0
    fi 
        
}


#make_rsrc () {
#    local name=${1}
#    local type=${2}
#    local cmd=${3}
#    
#    local i
#    for i in {1..100}
#    do
#    
#        cmd_out=`$cmd 2>&1`
#        
#        rsrc_exists ${name} ${type}
#        if [ $? -eq 1 ]; then
#            debug "Made rsrc $name $type"
#            
#            return 0
#        else
#            debug "Retry making rsrc $name $type"
#            sleep 5       
#        fi
#    done
#    return 1
#}

rsrc_exists () {
    local name=${1}
    local type=${2}
    
    present=`LC_ALL=C lsrsrc -s "Name = '$name'" $type | grep ResourceType `
    if [ -n "$present" ]; then
        return 1
    else
        return 0
    fi 
        
}

add_to_rg () {
    local name=${1}
    local type=${2}
    local group=${3}
    local order=${4}

    if [ -n "$order" ]; then
    	local order_flag="-p $order"
    fi
    
    local i
    for i in {1..100}
    do

        cmd_out=`LC_ALL=C addrgmbr $order_flag -g $group $type:$name 2>&1`        
        if [ $? -eq 0 ]; then
            debug "addrgmbr for $name worked."
            
            return 0
        else
            debug "addrgmbr output:$cmd_out"
            debug "Retry adding $name to $group, order:$order_flag"
            sleep 5       
        fi
    done
    return 1
}

chrg_l_none () {
    local rg=${1}

    local i
    for i in {1..100}
    do

        #local cmd_out=`chrg -l None $rg 2>&1`
        debug "tsa call hook 4"
        local cmd_out
        cmd_out=$(call_tsa_chrg_o_l_u -l None $rg)
        local cmd_rc=$?        
        if [ $cmd_rc -eq 0 ]; then
            debug "chrg -l None $rg -worked."
            
            return 0
        else
            debug "Failed:chrg -l None $rg :: rc:$cmd_rc output:$cmd_out"
            sleep 5       
        fi
    done
    return 1
}


get_ipv4_nic_address () {
    local nic=${1}
    #inet addr:9.10.111.244  Bcast:9.10.111.255  Mask:255.255.255.0
    ADDRESS=`LC_ALL=C ifconfig $nic | grep "inet addr" | awk '{print $2}' | cut -d: -f2`    
}

get_ipv6_nic_address () {
    local nic=${1}
    #inet6 addr: fe80::221:86ff:fe1a:3e1e/64 Scope:Link
    ADDRESS=`LC_ALL=C ifconfig $nic | grep "inet6 addr" | awk '{print $3}' | cut -d/ -f1`    
}


log_command_success () {
    
     log_command success
}

log_command_failed () {
    
     log_command failed
}


log_command () {
    
#     /opt/ibm/director/vmi/bin/logevent -t success | failed -c command name [-a command argument    
     /opt/ibm/director/vmi/bin/logevent -t $1 -c $EXTPROGNAME -a "$ORIG_PARMS"
     
}

is_automation_on() {
	
    local is_on=`LC_ALL=C lssamctrl Automation | grep Automation | awk '{print $3}'`
    if [ $? -eq 0 ]; then
        return $is_on
    else
        # Command didn't complete successfully
        return 99
    fi 
	
}

turn_on_automation () {
    #local result=`samctrl -M F 2>&1`
    debug "tsa call hook 5"
    local result
    result=$(call_tsa_samctrl_M_r F)
    debug "turn on automation: $result"
}

turn_off_automation () {
    #local result=`samctrl -M T 2>&1`
    debug "tsa call hook 6"
    local result
    result=$(call_tsa_samctrl_M_r T)
    debug "turn on automation: $result"
}

create_automation_flagfile () {
    touch $AUTOMATION_FLAGFILE
    
}

delete_automation_flagfile () {
    rm -f $AUTOMATION_FLAGFILE
    
}

add_ha_to_inittab () {
	
	# add the line if it doesn't exist
	local hainittab=`grep sha /etc/inittab | grep $SMHA_DIR`
	if [ -z "$hainittab" ]; then 
	   echo "sha:35:once:/opt/ibm/director/ha/bin/priv/smhamaint.sh #HA" >> /etc/inittab
	fi
	
	# uncomment it if it happens to exist
	uncomment_out_ha_inittab
}

comment_out_ha_inittab () {
	    perl -pi -e "s/^sha:35/#sha:35/g" /etc/inittab	
    
}
uncomment_out_ha_inittab () {
        perl -pi -e "s/^#sha:35/sha:35/g" /etc/inittab    
}

set_single_node_mode () {
    debug "setting single node mode"
    
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    
    chrsrc_out=`LC_ALL=C chrsrc -s "Name like '%' && ResourceType=1" IBM.Application NodeNameList="{'$rsct_node'}" 2>&1`
    debug "chrsrc out: $chrsrc_out"
}

set_dual_node_mode () {
    debug "setting dual node mode"
    
    get_value_from_cfg node-order
    node_order=$CFG_RETURN_VALUE    
    #debug "node_order is:$node_order"
    #hosts_list=`echo "'$node_order'" | sed "s/,/','/g"`
    
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    get_remote_rsct_node_name
    local remote_rsct_node=$REMOTE_RSCT_NODE_NAME
    
    if [[ "$host_list" == ${rsct_node},* ]]; then
    	hosts_list="'$rsct_node','$remote_rsct_node'"
    else
        hosts_list="'$remote_rsct_node','$rsct_node'"
    fi
    
    debug "hostlist is:$hosts_list"
    chrsrc_out=`LC_ALL=C chrsrc -s "Name like '%' && ResourceType=1" IBM.Application NodeNameList="{$hosts_list}" 2>&1`
    debug "chrsrc out: $chrsrc_out"
}

is_single_node_mode () {
    
    local dual_mode=`LC_ALL=C lsrsrc -s "Name like 'director' && ResourceType=1" IBM.Application NodeNameList | grep NodeNameList | grep ,`
    if [ -z "$dual_mode" ]; then
        return 1
    else
        return 0
    fi
}



create_exclude_peer_flagfile () {
    touch $EXCLUDE_PEER_FLAGFILE
    
}

delete_exclude_peer_flagfile () {
	debug "delete exclude node flag" 
    rm -f $EXCLUDE_PEER_FLAGFILE
    
}

FLOAT_RGS="$STORAGE_RG $DB2_RG $DIRECTOR_RG"
exclude_node_from_float_rgs () {
    local ex_node=${1}
    local ex_action=${2}
    
    local list_of_rgs=$FLOAT_RGS
    # Add the dhcp rg if it's shared configuration
    get_value_from_cfg dhcp
    case $CFG_RETURN_VALUE in
    "shared" )
        list_of_rgs="$FLOAT_RGS $DHCP_RG"
    ;;
    esac    
    
    local rg
    for rg in $list_of_rgs
    do
        debug "$ex_action $ex_node rg: $rg"
        
        is_rg_created $rg
        if [ $? -eq 1 ]; then
        
            lock_rg $rg
            	        
	        local max_tries=25
	        local count=1
	        local chrg_rc
	        while [ $count -le $max_tries ]
	        do
		        if [[ "$ex_action" == "exclude" ]]; then
		            #chrg_out=`chrg -u a -x $ex_node $rg 2>&1`
                            debug "tsa call hook 7"
		            chrg_out=$(call_tsa_chrg_o_l_u -u a -x $ex_node $rg)
		        else
		            #chrg_out=`chrg -u d -x $ex_node $rg 2>&1`
                            debug "tsa call hook 8"
		            chrg_out=$(call_tsa_chrg_o_l_u -u d -x $ex_node $rg)
		        fi
		        chrg_rc=$?
		        debug "Try $count chrg rc: $chrg_rc out: $chrg_out"
		        if [ $chrg_rc -eq 0 ]; then
		        	break
		        fi
		        # If it wasn't excluded in the first place, skip it.
                if [ $chrg_rc -eq 6 ]; then
                    if [[ "$ex_action" == "include" ]]; then
                        break
                    fi
                fi
		        
                let "count = count + 1"
	        
            done
            unlock_rg $rg
        else
            debug "RG $rg not excluded since it doesn't exist."
        fi
                
    done
    
    local exc_list=`LC_ALL=C lsrg -s "Name like '%-rg'" | grep -e Excl -e Name`
    debug "Excluded List: $exc_list"
    
    
}

lock_rg () {
	local rg=${1}
    
    local lock_rc=
    local rgreq_out=
        
    local max_tries=25
    local count=1
    local chrg_rc
    while [ $count -le $max_tries ]
    do    
	    rgreq_out=`LC_ALL=C rgreq -o lock $rg 2>&1`
	    lock_rc=$?
	    debug "rgreq rc: $lock_rc out: $rgreq_out"
	    
	    local lock_success=`echo $rgreq_out | grep "returned Token"`
	    if [ -n "$lock_success" ]; then
	    	break;
	    fi
        debug "Lock attempt on $rg failed. $lock_success"
        let "count = count + 1"    
    done
}

unlock_rg () {
    local rg=${1}
    local lock_rc=
    local rgreq_out=
    
    local max_tries=25
    local count=1
    local chrg_rc
    while [ $count -le $max_tries ]
    do
	    rgreq_out=`LC_ALL=C rgreq -o unlock $rg 2>&1`
	    lock_rc=$?
	    debug "rgreq rc: $lock_rc out: $rgreq_out"
        local lock_success=`echo $rgreq_out | grep "returned Token"`
        if [ -n "$lock_success" ]; then
            break;
        fi
        debug "Unlock attempt on $rg failed. $lock_success"
        let "count = count + 1"
    done
	
}

is_excluded_list_empty () {
	
	local lsrg_out=`LC_ALL=C lsrg -s "Name like '%'" | grep ExcludedList | awk '{print $3}' | grep -v {}`
	debug "lsrg out:$lsrg_out"
	if [ -z "$lsrg_out" ]; then
		return 1
	else
	    return 0
	fi
}

exclude_peer_node () {
    get_remote_rsct_node_name    
    exclude_node_from_float_rgs $REMOTE_RSCT_NODE_NAME exclude   
}

include_peer_node () {
    get_remote_rsct_node_name
    exclude_node_from_float_rgs $REMOTE_RSCT_NODE_NAME include   
}

include_this_node () {
    get_local_rsct_node_name
    exclude_node_from_float_rgs $LOCAL_RSCT_NODE_NAME include   
}

include_all_nodes () {
    include_peer_node
    include_this_node   
}

exclude_node_from_domain () {
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    
	domain_excluded_node $rsct_node exclude
}
include_node_in_domain () {
	get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
	
	domain_excluded_node $rsct_node include
}

domain_excluded_node () {
    local ex_node=${1}
    local ex_action=${2}
    
    debug "domain_excluded_node node: $ex_node action: $ex_action"        
        
    
    local max_tries=25
    local count=1
    while [ $count -le $max_tries ]
    do
        if [[ "$ex_action" == "exclude" ]]; then
            sam_out=`LC_ALL=C samctrl -u a $ex_node 2>&1`
        else
            sam_out=`LC_ALL=C samctrl -u d $ex_node 2>&1`
        fi
        local sam_rc=$?
        debug "Try $count samctrl rc: $sam_rc out: $sam_out"
        # See if the command worked by looking at the exlude list        
        local node_present=`LC_ALL=C lssamctrl 2>&1 | grep ExcludedNodes | grep $ex_node`
        local it_worked=0
        
        if [[ "$ex_action" == "exclude" ]]; then
        	if [ -n "$node_present" ]; then
        		it_worked=1
        	fi
        else
            if [ -z "$node_present" ]; then
                it_worked=1
            fi
        fi

        if [ $it_worked -eq 1 ]; then
            break
        else
          let "count = count + 1"
          sleep 4
        fi
    
    done
    
    local exc_list=`LC_ALL=C lssamctrl 2>&1`
    debug "samctrl Excluded List: $exc_list"
	
}

create_shutdown_halt_flagfile () {
    touch $SHUTDOWN_HALT_FLAGFILE    
}
delete_shutdown_halt_flagfile () {
    rm -f $SHUTDOWN_HALT_FLAGFILE    
}

create_stop_maint_flagfile () {
    touch $STOP_MAINT_FLAGFILE    
}
delete_stop_maint_flagfile () {
    rm -f $STOP_MAINT_FLAGFILE    
}

set_crit_rsrc_protection () {
	local level=$1
    local chrsrc_out=`LC_ALL=C chrsrc -c IBM.PeerNode CritRsrcProtMethod=$level 2>&1`
    debug "chrsrc -c IBM.PeerNode CritRsrcProtMethod=$level output:$chrsrc_out"
}

create_syncconf_flagfile () {
    touch $SYNCCONFIG_FLAGFILE
    
}

delete_syncconf_flagfile () {
    rm -f $SYNCCONFIG_FLAGFILE
    
}

create_refresh_managementIP_flagfile () {
    touch $REFRESH_MANAGEMENTIP_FLAGFILE
    
}

delete_refresh_managementIP_flagfile () {
    rm -f $REFRESH_MANAGEMENTIP_FLAGFILE
    
}

get_node_value () {
    get_value_from_cfg node
    node=$CFG_RETURN_VALUE                            
}

sync_tsa_config () {
    
    # get the node value from the config file.  This will ensure it's the latest
    get_node_value
        
    local cmd_out=`LC_ALL=C runact -c IBM.PeerDomain SyncConfig DomainName=$TSA_DOMAIN TargetNodes="{'$HOST','$node'}"`
    debug "SyncConfig output:$cmd_out"
}


add_etc_hosts_entry () {
	local address=${1}
	local hname=${2}
	 
	echo "$address $hname" >> /etc/hosts
	debug "add /etc/hosts entry: $address $hname"
	
}

ping_host () {
	local other_node=$1
	
	ping -c 5 -q $other_node 1> /dev/null 2>&1  
    RC=$?
    debug "ping $other_node rc:$RC"
    if [ $RC -ne 0 ]; then
        # since we can't do an IPv4 ping, try IPv6 ping
        ping6 -c 5 -q $other_node 1> /dev/nul 2>&1
        RC=$?
        debug "ping6 $other_node rc:$RC"
        
    fi
    
    return $RC
	
}

# I see that there was a get_host_address routine in smhaagent.sh  that checked DNS first before
# checking /etc/hosts.  I don't understand that but it is too late in the release for me to 
# feel comfortable changing that.  This is sort of the same routine.  It lets "ping" find the address
# and ping uses /etc/hosts first, and then DNS next...

get_host_address_etc_hosts_override () {
    local name=${1}
        
    HOST_ADDRESS=

    # PING pfeecl3lp5.in.ibm.com (9.126.134.214) 56(84) bytes of data.
    
    local pcmd=`ping -c 1 $name | grep PING`
    debug "get_host_address_etc_hosts_override: ping output: $pcmd"
    local h1=${pcmd#*(}
    local h2=${h1%%)*}
    debug "h2:$h2"
    HOST_ADDRESS=$h2        
       
 	debug "get_host_address_etc_hosts_override: $name $HOST_ADDRESS"
 	
    if [ -z "$HOST_ADDRESS" ]; then
   
        fatal_error "`eval_gettext \"Could not resolve address for host name \\\$name\"`"
    fi
    
}

get_admin_id () {
	local get_admin=/opt/ibm/director/vmi/bin/lsdefaultadmin
	
	if [ -n "$ADMIN_ID" ]; then
		return
	fi 
	
	if [ -x "$get_admin" ]; then
		local admin_id=`$get_admin`
	else
	    local admin_id=sysadmin
	fi
	
	ADMIN_ID=$admin_id
	
}

get_cfg_user_id () {
	
    if [ -n "$CFG_USER_ID" ]; then
        return
    else
        if [ -e $SMHA_CONFIGDIR/.cfg_user ]; then
        	CFG_USER_ID=`cat $SMHA_CONFIGDIR/.cfg_user`
        else
            get_admin_id
            CFG_USER_ID=$ADMIN_ID
        fi
    fi     
}

save_cfg_user_id () {
	local user_id=$1
	
	echo "$user_id" > $SMHA_CONFIGDIR/.cfg_user
	CFG_USER_ID=$user_id
}

rm_cfg_user_file () {
	rm -f $SMHA_CONFIGDIR/.cfg_user
}


record_cfg_status () {
	local status=$*
	
	echo "$status" >> $CFG_STATUS_FILE
	run_init "status" "$status"
		
}

rm_cfg_status_file () {
	rm -f $CFG_STATUS_FILE
}

is_passive () {
    local name
    local opstate
    local node
    local resource
    local is_passive=Y

    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
            
    local tmpf=`mktemp /tmp/smha_ip.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
#    lsrsrc IBM.ServiceIP Name OpState NodeNameList ResourceType > $tmpf
    LC_ALL=C lsrsrc -s 'Name like "drbdp%"' IBM.Application Name OpState NodeNameList ResourceType 2>/dev/null 1> $tmpf
    # Set a variable saying we haven't read any lines.  We use this to see if the file was empty.
    readaline=0
    while read -r line
    do
        # We were able to read at least one line.
        readaline=1
        case "$line" in
#            resource*)
#                debug "resource"
#           ;;
            Name*)
                 name=`echo $line | cut -d' ' -f 3`
#                 debug "name: $name"   
            ;;
            OpState*)
                 opstate=`echo $line | cut -d' ' -f3`
#                 debug "opstate: $opstate"
            ;;
            NodeNameList*)
                 node=`echo $line | cut -d' ' -f3`
#                 debug "node: $node"                 
            ;;
            ResourceType*)
                 resource=`echo $line | cut -d' ' -f3`
                 # Resource is the last attribute in the groupings so process the group:
                 
                 # Only process fixed resources (floats have fixed too!)
                 if [ "$resource" == "0" ]; then
                    # Next see if this node is for this host
                    grepstr=`echo $node | grep $rsct_node`
#                    debug "Status Grep str:$grepstr"
                    if [ -n "$grepstr" ]; then
                        case $opstate in
                            # 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=stuck online
                            1|5)
#                                 debug "Opstate is: $opstate"
                                 # This node is not passive
                                 is_passive=N
                            ;;
                        esac
                    fi
                 fi
            ;;
        esac
    done < $tmpf    
    rm $tmpf

    # Check if we had trouble creating the file contents and couldn't read a single line.
    if [ $readaline -eq 0 ]; then
        fatal_error "`eval_gettext "Could not make temporary file"`"
    fi
    
    # If this node is not passive then return 0
    if [ "$is_passive" == "N" ]; then
        
        return 0
    else
        return 1                
    fi
    
    
}

get_local_rsct_node_name () {
	
	if [ -z "$LOCAL_RSCT_NODE_NAME" ]; then
	   local node_name=`LC_ALL=C lsrpnode -L | grep -v RSCTVersion | awk '{print $1}'`
	   LOCAL_RSCT_NODE_NAME=$node_name
	   debug "LOCAL_RSCT_NODE_NAME:$LOCAL_RSCT_NODE_NAME"
	fi
}

get_remote_rsct_node_name () {
    
    if [ -z "$REMOTE_RSCT_NODE_NAME" ]; then
	    get_local_rsct_node_name
	    local local_node_name=$LOCAL_RSCT_NODE_NAME
	    
	    local node_name=`LC_ALL=C lsrpnode | grep -v RSCTVersion | grep -v "^${local_node_name} " | awk '{print $1}'`
	    REMOTE_RSCT_NODE_NAME=$node_name
	    debug "REMOTE_RSCT_NODE_NAME:$REMOTE_RSCT_NODE_NAME"
    fi
}


MUTEX_DIR=$SMHA_CONFIGDIR/.mutex
mutex_on () {
	local lock=${1}
	local lockflag=MUTEX_${lock}
	local mutex_dir=${MUTEX_DIR}/${lock}
	
	mkdir -p `dirname $mutex_dir`  
    debug "mutex $lock on"
    if [ -z "${!lockflag}" ]; then
        while true; do
            mkdir $mutex_dir 1>/dev/null 2>&1
            mkdir_rc=$?
            if [ $mkdir_rc -eq 0 ]; then
            	eval ${lockflag}=on
                debug "mutex $lock successfully locked"
                return
            else
                sleep 4
                debug "mutex $lock already locked, sleeping. rc=$mkdir_rc"
            fi
            
        done
    fi
}

mutex_off () {
    local lock=${1}
    local lockflag=MUTEX_${lock}
    local mutex_dir=${MUTEX_DIR}/${lock}
	
    if [ -n "${!lockflag}" ]; then
        debug "mutex $lock off"     
        rmdir $mutex_dir
    fi
    eval ${lockflag}=
}

mutex_clear_all () {
	if [ -d $MUTEX_DIR ]; then
	   rm -rf $MUTEX_DIR
	fi
}


