# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.


delete_tsa_model () {
    
    debug "Deleting the tsa model."

    is_domain_setup
    if [ $? -eq 0 ]; then
        if [ -f $SMHA_CFG_FILE ]; then
            # if our config file still exists just erase it and exit
            sm_msg "`eval_gettext "The HA configuration appears to be already terminated."`"
            sm_msg "`eval_gettext "Removing configuration file."`"
            
            rm $SMHA_CFG_FILE
            return
        else      
            # Otherwise, neither the domain nor our config file were there.     
            sm_msg "`eval_gettext "The HA environment has not been setup."`"
            return
        fi
    fi
        
    ### Delete the model!
    
    take_rgs_offline $DIRECTOR_RG $DB2_RG $DHCP_RG $STORAGE_RG $DRBD_RG
           
    take_all_rg_offline
    
    run_plugins 6 delete
    
    is_rg_created $DIRECTOR_RG
    if [ $? -eq 1 ]; then
        sm_msg "`eval_gettext "Waiting for Director to shutdown."`"
        take_rg_offline_n_wait_nonfatal $DIRECTOR_RG
    fi
             
    sm_msg "`eval_gettext "Removing the resources."`"
    
    run_plugins 4 delete

    sm_msg "`eval_gettext "Removing DB2 resources."`"                           
    dlt_db2_rsrc        

    run_plugins 2 delete
    
    sm_msg "`eval_gettext "Removing storage resources."`"
    dlt_drbd_rsrc
    
    run_plugins 0 delete
        
    sm_msg "`eval_gettext "Removing the domain."`"
    delete_domain
                
    sm_msg "`eval_gettext "Director High Availability setup was deleted."`"
        
}

crt_tsa_model () {
    
    # Do the setup
    sm_msg "`eval_gettext "Creating the domain."`"
    debug "calling mkrpdomain $TSA_DOMAIN $HOST $node"
    mkrpdomresult=`LC_ALL=C mkrpdomain $TSA_DOMAIN $HOST $node 2>&1`
    local mkrpRC=$?
    debug "mkrpdomain rc:$mkrpRC: $mkrpdomresult"
    if [ $mkrpRC != 0 ]; then
        #mkrpdomain might have failed because the environment was IPv6 only, so try again with -6
        mkrpdomresult=`LC_ALL=C mkrpdomain -6 $TSA_DOMAIN $HOST $node 2>&1`
        mkrpRC=$?
        debug "mkrpdomain -6 rc:$mkrpRC: $mkrpdomresult"
    fi
    
    # Wait for a little bit so it can get propagated to the nodes
    sleep 5
    
    sm_msg "`eval_gettext "Starting the domain."`"
    record_cfg_status "`eval_gettext "Starting the domain."`"
    
    start_n_wait_for_domain
    
    # Turn on automation, so things come online
    debug "Turning on automation"                 
    #result=`samctrl -M F -r $TSA_RETRY_COUNT 2>&1` 
    debug "tsa call hook 10"
    result=$(call_tsa_samctrl_M_r F $TSA_RETRY_COUNT)
    local sam_rc=$?        
    debug "samctrl result:$result"
    if [ $sam_rc -ne 0 ]; then
        fatal_error "`eval_gettext \"Could not turn on automation. rc:\\\$sam_rc\"`"
    fi
           
    # Touch the config file so the monitors start reporting status
    touch $SMHA_CFG_FILE
                                    
    ### Create the model ###
    
    sm_msg "`eval_gettext "Creating the resources."`"
    
    run_plugins 0 create
    
    # DHCP Must be done when the storage is online 
    # on both nodes.  We may need to copy off the config   
    # Model DHCP if it's on
    record_dhcp_autostart
    crt_dhcp_rsrc_if_on 
    
    # Always turn off DHCP
    remove_dhcp_autostart
    
    record_cfg_status "`eval_gettext "Creating storage resources."`"            
    crt_all_drbd_rsrcs
        
    run_plugins 2 create
            
    # Create the DB2 resource in TSA
    record_cfg_status "`eval_gettext "Creating database resources."`"
    crt_db2_rsrc

    # Create the DB2 dependencies        
    crt_db2_dep 

    run_plugins 4 create
    
    record_cfg_status "`eval_gettext "Creating CIM resources."`"
    crt_cim_rsrcs

    # Make sure director is not setup to autostart
    remove_director_autostart
    
    # Create the Director resource
    record_cfg_status "`eval_gettext "Creating Director resources."`"    
    crt_dir_rsrc
                                    
    # Create Director dependencies
    crt_dir_dep

    # Create floating IPs
    record_cfg_status "`eval_gettext "Creating networking resources."`"    
    crt_network_aliases
    
    run_plugins 6 create

    # Make sure ntp is not setup to autostart
    #  --- doesn't matter since TSA will just insure it's running.
#    remove_ntp_autostart

    # Create the NTP resource
    record_cfg_status "`eval_gettext "Creating NTP resources."`"    
    crt_ntp_rsrc
    
    # Create the Quorum Tie Breaker
    if [ -n "$network_quorum" ]; then
        debug "Create Network Quorum from: $network_quorum"
        crt_network_quorum $network_quorum
    else
        crt_yes_quorum
    fi

    # Start the base drbd rg
    record_cfg_status "`eval_gettext "Starting storage resources."`"    
    start_n_wait_for_rg_online $DRBD_RG
                                        
    # We need the storage online 
    ch_rg_online $STORAGE_RG
    
    if [ $DRBD_EQU -eq 0 ]; then
	    # Wait while drbd is syncing. It takes a while to start syncing.
	    sleep 240    
	    wait_while_syncing
    fi
    
    wait_for_rg_online $STORAGE_RG
    
    # Do the second part of the dhcp setup
    crt_dhcp_rsrc_step2
    
    # Update the agent files with the agent address
    record_cfg_status "`eval_gettext "Configuring Agent Manager."`"    
    updt_all_agntmgr_files_primary
            
}

delete_domain() {
    
    # The domain must be online in order to delete it.
    
    debug "Start the domain"
    start_n_wait_for_domain
            
    take_all_rg_offline
    sleep 5
    wait_all_rg_offline
    
    rm_all_rg
    
    debug "Remove the domain"
    remove_n_wait_for_domain    
    
    rc=$?
    if [ $rc -ne 0 ]; then
        fatal_error "`eval_gettext \"Timed out waiting for TSA domain to be deleted. rc=\\\$rc\"`"
    fi
    
}

