#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996 
#
# (C) Copyright IBM Corp. 2010, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smharsrc.sh

wait_for_sam () {

    while true; do
		
	    local ctrl_out=`LC_ALL=C lssamctrl 2>&1`
	    ctrl_rc=$?
	
	    if [ $ctrl_rc -eq 0 ]; then
	        debug "HA: $ctrl_out"
	        local automation=`echo $ctrl_out | grep Automation`
	        if [ -n "$automation" ]; then        
	            return
	        else
	            debug "Waiting for sam: rc=$ctrl_rc"
	            sleep 10
	        fi
	    else
	        debug "Waiting for sam: rc=$ctrl_rc"
	        sleep 10
	    fi

    done
}

wait_for_director_rg () {
	
	local pending=0
    for i in {1..96}
    do
        get_rg_status_num $DIRECTOR_RG
        # 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline        
        case $? in
        1|4 )
            return 0
        ;;        
        5 )
            pending=1
            sleep 5
        ;;
        3 )
            # If it's failed offline and we previously saw it starting, then return
            if [ $pending -eq 1 ]; then
            	return 0
            else
                sleep 5
            fi
        ;;
        *)
        # Otherwise, just wait until possibly our time out
            sleep 5
        ;;
        esac
    done
	
}


hdl_cluster_version_updates () {
	
	run_init clustervrm
	RC=$?
	return $RC
}

###################################
## Main code                     ##
###################################
RC=0
debug "smhamaint called"

#if the stop maint flag is on, then don't run smhamaint
if [ -e $STOP_MAINT_FLAGFILE ]; then
    delete_stop_maint_flagfile
    debug "stop maint flag is on...so exiting smhamaint"
    exit 0
fi

mutex_clear_all

# Do not use is_configured because at this point (when the appliance
# is being started up), is_configured can sometimes return false
# even when HA really is configured.  Instead use the code below.
isHAConfigured=0
if [ -f $SMHA_CFG_FILE ]; then
    # Config file is there but is the node parameter present?
    get_value_from_cfg node
    if [ $? -eq 0 ]; then
        isHAConfigured=1
    fi
fi

if [ $isHAConfigured -eq 0 ]; then
	debug "HA is not configured."
else

    # Reset the dirc settings to passive, they'll get set to active if director is started.
    dirc_use_settings passive
    
    wait_for_sam
    
    # check on every IPL - if RSCT or TSA version has moved forward, then process appropriately
    hdl_cluster_version_updates
    
    # if automation flag file is found, then handle it.
    if [ -e $AUTOMATION_FLAGFILE ]; then
    	is_automation_on
    	if [ $? -eq 0 ]; then
    		debug "Automation is not on, going into single node mode"
	    	set_single_node_mode 
	    	turn_on_automation
	    	delete_automation_flagfile
	    	# Wait enough time to get the resources starting on the desired node
	    	sleep 120
	    	wait_for_director_rg
	    	set_dual_node_mode
    	else
            # Automation is already on, so just delete the flag and ignore it.
            debug "Automation already turned on."
            delete_automation_flagfile    	   
        fi
    fi
        
    # if the shutdown halt flag is on, then handle it.
    if [ -e $SHUTDOWN_HALT_FLAGFILE ]; then
        debug "shutdown halt flag is on...handling it"
        
        include_node_in_domain
#        set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET
        delete_shutdown_halt_flagfile
    fi

    # if the SyncConf is needed.  This must be done after we include this node
    # in the domain since we post a msg to our mq.
    #if [ -e $SYNCCONFIG_FLAGFILE ]; then
        # This is needed when the ip address changes and the system is rebooted
        # Send the sync request to the other node.
           # sync_tsa_config
           
    #    debug "SyncConfig"
    #    run_init "SyncConfig" 
        
    #    delete_syncconf_flagfile
    # fi
    
    # if the refreshManagementIP flag is on
    if [ -e $REFRESH_MANAGEMENTIP_FLAGFILE ]; then
    	# an IP address was changed via chnetcfg.  There could have 
    	# been an "add" of a global IPv6 address on one of the interfaces.
    	# Ensure that we heartbeat on all IPv6 addresses that are a global scope
    	turn_on_ipv6_global_heartbeat
    	chgrc=$?
    	# wait for director to come online, 
   	 	# and then refresh management IPs as needed.
   		wait_for_director_rg
    	refresh_managementIPs
    	# delete the file now
    	if [ $chgrc -eq 0 ]; then
    		delete_refresh_managementIP_flagfile
    	fi
    fi 
    
    # if the exclude peer node flag is on, then handle it.
    if [ -e $EXCLUDE_PEER_FLAGFILE ]; then
        debug "Exclude flag is on...handling it"
        
        # Wait enough time to get the resources starting on this node
        # ...as long as the other node didn't exclude us...
        
        were_exc=`LC_ALL=C lsrg -s "Name like '%-rg'" 2>&1 | grep ExcludedList | grep $HOST`
        if [ -z "$were_exc" ]; then
            sleep 120
            wait_for_director_rg
        else
            debug "Found ourselves being excluded by peer."
        fi
        
        include_peer_node
#        set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET
        
        delete_exclude_peer_flagfile
    fi

    # Always restore the resource protection.  If we're the only node up then it'll fail and
    # the protection will still be off (which is what we want.)
    set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET
    
fi


debug "smhamaint exit; RC=$RC"
exit $RC
