#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

#### Standard code to find an pull in all sm common code ####
# Use the specified path to this command, then working directory, then path
PATH=$PROGPATH:`pwd`:$PATH
CMDFINDER=which
CMD_DIR=`$CMDFINDER $PROGNAME`
CMD_DIR=`dirname $CMD_DIR`

COMMON_CODE=$CMD_DIR/../include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh

DIR_LOCK_FILE=/var/lock/subsys/lwi


wait_for_start () {
    local rc=0

    local saw_pid=0
	# The Director start command is asynchronous.  Wait until we have an indication that Director is starting
	local loop_period=7
	# Calculate how many times we should loop based on our timeout value
	
	# declare an integer variable, calculate the max loops. 
	# Subtract out some periods to account for overhead, so we don't time out.
	declare -i loop_max
	declare -i loop_third
	let "loop_max=($DIR_MON_START_TIMEOUT/$loop_period)-10"
	let "loop_third=($loop_max/3)"
		
	debug "Director wait loop max: $loop_max"
	local i=1
	while [ $i -ne $loop_max ]
	do
		# Test if we're terminating and jump out of the loop
		is_terminating
		if [ $? -eq 1 ]; then
			debug "Director start loop stopped because of terminating flag"
			return $rc
		fi
		
		is_mounted /$DB2_MIRROR_NAME
        if [ $? -ne 1 ]; then
            debug "Director start loop stopped because mirror disk not mounted"
            return ${TSA_OFFLINE}
        fi
		
		get_status
	    local statusrc=$?
	    debug "Wait for Director start, count $i of $loop_max, director status rc:$statusrc"
	    case $statusrc in
	    	# If the status shows signs of starting then end looping
			$TSA_ONLINE ) 
#				debug_tsa "Wait for Director start, hit break"
                rc=0
				break
			;;
			$TSA_PENDING_ONLINE)
			    # It seems to be coming up...looking good so far
			    # Mark that we saw the PID because we know it's starting
			    saw_pid=1
			    rc=0
			;;
			*) 
			    # No signs of starting...check for errors
			    get_dir_pid
			    local isDirectorUp=$?
			    if [ $isDirectorUp -ne 0 ]; then
    			    debug "Director process still exists: $isDirectorUp"
			        # The process is still running, record that we saw it
			        saw_pid=1
			    else
			        # We didn't find the process, have we ever seen it?
			        if [ $saw_pid -ne 0 ]; then
			            debug "Director process no longer exists"
			            return ${TSA_OFFLINE}
			        fi
			        # Check how long we've been looping
			        if [ $i -gt $loop_third ]; then
			            # It's been past a third of the time and we've seen no signs of starting.
			            debug "Hit 1/3 time out."
			            return ${TSA_OFFLINE}
			        fi
			        
			    fi
			;;
		esac
		sleep $loop_period	
		let "i = i +1"			
	done
	debug_tsa "Wait for Director start done: dir status:$statusrc rc:$rc"
	return $rc
}

get_status () {
	local RC
	
    running=`$STATUS_CMD`
    local statusrc=$?
    case $statusrc in
        0) #Active
            RC=${TSA_ONLINE}
            ;;
        1) #Starting
            RC=${TSA_PENDING_ONLINE}
            ;;
        2) #Ending
            RC=${TSA_PENDING_OFFLINE}
            ;;
        3) #Inactive

            get_dir_pid
            # If the director process is not found...
            if [[ $? -eq 0 ]]; then
                RC=${TSA_OFFLINE}
            else
                # Otherwise the process is running, but we don't know why it's inactive.
                RC=${TSA_UNKNOWN}
            fi

            ;;
        4) #Error
        
            get_dir_pid
            # If the director process is not found...
            if [[ $? -eq 0 ]]; then
                RC=${TSA_OFFLINE}
            else
                # Otherwise the process is running, but we don't know why it's in error.
                RC=${TSA_UNKNOWN}
            fi
            ;;
        *)
            RC=${TSA_UNKNOWN}
            debug "Director status code unknown. ReturnCode:$statusrc"
            ;;
    esac
    debug_tsa "Director get_status rc=$RC"
    return $RC
}


get_dir_pid () {
	# Don't count on the return code value!!! Use global DIRECTOR_PID variable!
    
    DIRECTOR_PID=`ps auxww | grep LaunchLWI |  grep -v grep | grep -v -i agent | grep -i "director/jre/bin"  | awk '{print $2}'`
    debug "get_dir_pid:Director pid is:$DIRECTOR_PID"
    if [ -n "$DIRECTOR_PID" ]; then
        return 1
    else
        debug "get_dir_pid: return 0"
        return 0
    fi    
    
}


really_stop () {

    declare -i loop_max
    loop_max=10
    local sig_nine=
    	
	while [ $loop_max -ne 0 ]; do
	    get_dir_pid
	    local isDirectorUp=$?
	    if [ $isDirectorUp -ne 0 ]; then
	        debug "Sleeping before killing director PID:$DIRECTOR_PID"
	        sleep 20
	        get_dir_pid
	        isDirectorUp=$?
	        if [ $isDirectorUp -ne 0 ]; then
	        	if [ $loop_max -eq 4 ]; then
	        		sig_nine="-9"
	        	fi
	            local kill_out=`kill $sig_nine $DIRECTOR_PID`
	            debug "Director kill output:$kill_out"
	        else
                return
	        fi
	    else
	        return
	    fi
	    let "loop_max=$loop_max - 1"
	done
}

check_for_mounts () {
	
	# Check if the director mounts are properly mounted.
	local mounts_out=`cat /proc/mounts | grep director/data | grep drbd`
    if [ -z "$mounts_out" ]; then
    	return 0
    else
        return 1
    fi
}

set_dir_max_heap_size() {
    # Set up JVM heap for Systems Director's JVM
    typeset -i totalmem
    typeset -i memsize
    totalmem=`cat /proc/meminfo |grep "MemTotal:"|awk '{print $2}'|tr -d " "`
    memsize=`echo $((totalmem/1024/1024))`
    debug "set_dir_max_heap_size, memsize = $memsize"
    if [ $memsize -lt 6 ]; then
        echo " -Xmx3g"   >  /opt/ibm/director/lwi/conf/overrides/maxheap.javaopt
    elif [ $memsize -ge 6 -a $memsize -lt 8 ]; then
        echo " -Xmx4g" >  /opt/ibm/director/lwi/conf/overrides/maxheap.javaopt
    elif [ $memsize -ge 8 -a $memsize -lt 16 ]; then
        echo " -Xmx6g" >  /opt/ibm/director/lwi/conf/overrides/maxheap.javaopt
    else
        echo " -Xmx8g" >  /opt/ibm/director/lwi/conf/overrides/maxheap.javaopt
    fi
}

#### Main Code ##############################################
START_CMD=$INSTALL_DIR/bin/smstart
STOP_CMD=$INSTALL_DIR/bin/smstop
#STATUS_CMD=$INSTALL_DIR/bin/smstatus
STATUS_CMD=$INSTALL_DIR/ha/bin/tsa/smhadirstatus.sh

RC=$TSA_UNKNOWN  

Action=${1}

case ${Action} in
start)
    record_tsa_action $Action director "-"
    
    check_for_mounts
    if [ $? -eq 1 ]; then    
	    # Make sure the HA property values are set
	    set_dir_ha_props
	    
	    # Set director JVM max heap size
	    set_dir_max_heap_size
	    
	    sm_sys_log "IBM Director starting"
	    
	    # Pause drbd syncing
	    pause_mirror_sync "director-starting"
	    
	    if [ -e $DIR_LOCK_FILE ]; then
	        debug "Director lock file found during start"
	        rm -f $DIR_LOCK_FILE
	    fi
	    
	    # The start command is asynchronous
	    running=`$START_CMD 2>&1`
	    debug "$running"
	    
	    locked=`echo "$running" | grep "$DIR_LOCK_FILE"`    
	    if [ -n "$locked" ]; then
	        locked=`echo "$running" | grep "unexpectedly failed"`
	    fi    
	    if [ -n "$locked" ]; then
	    	debug_tsa "Director lock file found.  Removing."
	    	# Delete the lock file
	    	rm $DIR_LOCK_FILE
	    	# Try starting it again
		    running=`$START_CMD 2>&1`
		    debug "$running"    	
	    fi

        # Set the active dirc settings and kill local browser connection to Director
        get_dirc_browser_pid    
        dirc_use_settings active
        kill_dirc_browser
	    
		wait_for_start	
		RC=$?
		if [ $RC -eq 0 ]; then
	        sm_sys_log "IBM Director started"
	    else
	        sm_sys_log "IBM Director start -failed"
	    fi
	    
	    # resume syncing
	    /sbin/drbdadm resume-sync all
	    
    else
        RC=99    
    fi 
    
    ;;
stop)
    record_tsa_action $Action director "-"
    sm_sys_log "IBM Director stopping"
    
    if [ -z "$SA_RESET" ]; then
    	SA_RESET=0
    fi
    # If this is a second call to stop director...
    if [ $SA_RESET -eq 1 ]; then
    	debug "SA_RESET is set, taking down director hard."
    	get_dir_pid
    	local isDirectorUp=$?
    	if [ $isDirectorUp -ne 0 ]; then
            local kill_out=`kill -9 $DIRECTOR_PID`
            debug "Director kill output:$kill_out"
    	fi    	           
    else
	    # The stop command is synchronous
	    $STOP_CMD
	    really_stop    	
    fi
    
    # Set the passive dirc settings and kill local browser connection to Director
    get_dirc_browser_pid    
    dirc_use_settings passive
    kill_dirc_browser
    
    
    sm_sys_log "IBM Director stopped"
    
    RC=0
    ;;

status)
    get_status
    RC=$?
    
    # Pause drbd sync based on Director status
    case $RC in
    	$TSA_ONLINE            ) resume_mirror_sync "director-starting";;
        $TSA_OFFLINE           ) resume_mirror_sync "director-starting";;
    	$TSA_PENDING_ONLINE    ) pause_mirror_sync "director-starting";;
    	$TSA_PENDING_OFFLINE   ) resume_mirror_sync "director-starting";;
    	# don't do anything for UNKNOWN
    esac        
    ;;
cleanup)
    debug "Director failed."
    ;;    
esac
record_tsa_action $Action director $RC

exit $RC
