#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
EXTPROGNAME="smha"

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help s start p stop S suspend R resume r reset F failover v validate c cancel T tsa M meps i immediate"
PARM_MULTI=""
PARM_TRANSLATION="h help \
s start \
p stop \
S suspend \
R resume \
r reset \
T tsa \
M meps \
F failover \
v validate \
c cancel \
i immediate \
" 
PARM_SINGLES="help start stop suspend resume"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help start stop suspend resume reset failover validate cancel immediate"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh

###################################
## Functions                     ##
###################################
proc_parms () {
	
	# Parse the parameters
	# The result should be that variables are set for each parameter and
	# CMD_PARMS will contain the list of parameters processed
	# If a file is specified it will also read the file.
    parse_all_parms $*
    
	
    for option in $CMD_PARMS
    do
    	# For each option, make sure the minimum parameters are specified
	    case $option in 
	        validate)
	           	ensure_parms "failover"
	        ;;
	        "cancel")
	           	ensure_parms "failover"
	        ;;
	    esac
    done

    
}

echo_help () {
    echo -e "`eval_gettext \"\\\${EXTPROGNAME}: [-h|--help][-s|--start][-p|--stop][-S|--suspend][-R|--resume][-r|--reset][-F|--failover [ [-v|--validate] | [-c|--cancel] ] ]\"`"
    echo -e "`eval_gettext \"The \\\${EXTPROGNAME} command is used to manage High Availability for Director.\"`"
    echo -e "`eval_gettext "Options:\n"`"
    echo -e "`eval_gettext "\t-h,--help\tDisplay this help"`"
    echo -e "`eval_gettext "\t-s,--start\tStart HA"`"
    echo -e "`eval_gettext "\t-p,--stop\tStop HA"`"
    echo -e "`eval_gettext "\t-S,--suspend\tSuspend Director"`"
    echo -e "`eval_gettext "\t-R,--resume\tResume Director"`"
    echo -e "`eval_gettext "\t-r,--reset\tReset HA resources"`"
    echo -e "`eval_gettext "\t-F,--failover\tFailover to the passive node (unless -c or -v is specified)"`"
    echo -e "`eval_gettext "\t-v,--validate\tCheck for errors that will prevent the failover"`"
    echo -e "`eval_gettext "\t-c,--cancel\tAttempt to cancel a failover that is in progress\n"`"
}

#####################################################
# Process the help parameter
#####################################################
proc_help () {
	
	if [[ "$help" == "Y" ]]; then
		echo_help
        exit 0
	fi

}

chk_ha_setup () {
	is_domain_setup
    if [ $? -eq 0 ]; then
        fatal_error_rc $HA_ENVIRONMENT_NOT_CONFIGURED "`eval_gettext "The HA environment has not been setup."`"
    fi
	
}

#####################################################
# Process the stop parameter
#####################################################
proc_stop () {
    
    if [ -n "$stop" ]; then
        # The stop paramter was specified
        
        chk_ha_setup
                
        # We can't keep the state of "stopped" in the config.
        # That's because they could be doing it from either node and the cfg's would be out of sync
        # Instead just shutdown the domain
        
        take_all_rg_offline
        # Give them some time before we start polling if they are down
        sleep 15
        wait_all_rg_offline
        
        stoprpdomain $TSA_DOMAIN
        local lrc=$?
        if [ $lrc -eq 0 ]; then
        	sm_msg "`eval_gettext "Director HA stopped."`"
        else
            sm_log "Return code from stoprpdomain: $lrc"
        	fatal_error "`eval_gettext "An error occurred while stopping the Director HA setup."`"
    	fi
    	
    	log_command_success        
        exit 0
    fi
    
}

#####################################################
# Process the start parameter
#####################################################
proc_start () {
    
    if [ -n "$start" ]; then
        # The start paramter was specified
        
        chk_ha_setup
                
        # Just start the domain        
        start_n_wait_for_domain
        
        # Start all rg
        start_all_rg_online                   
                                
        sm_msg "`eval_gettext "Director HA started."`"
        log_command_success
        exit 0
    fi
    
}


#####################################################
# Process the suspend parameter
#####################################################
proc_suspend () {
    
    if [ -n "$suspend" ]; then
    	
        chk_ha_setup
                
        # We can't keep the state of suspend in the config.
        # That's because they could be doing it from either node and the cfg's would be out of sync
        
        take_rg_offline $DIRECTOR_RG
        sleep 5
        wait_for_rg_offline $DIRECTOR_RG
        
        sm_msg "`eval_gettext "Director HA suspended."`"
        log_command_success
        exit 0
    fi
    
}

#####################################################
# Process the resume parameter
#####################################################
proc_resume () {
    
    if [ -n "$resume" ]; then
        chk_ha_setup
                       
        # Start all rg
        start_all_rg_online                   
                                
        sm_msg "`eval_gettext "Director HA resumed."`"
        log_command_success
        exit 0
    fi
    
}

####################################################

in_move () {
	
    local in_move=`LC_ALL=C lssam | grep "$STORAGE_RG" | grep "Move"`
    if [ -n "$in_move" ]; then
        return 1               	
    fi
    
    return 0
	
}

all_mirrors_uptodate () {

    # Assume all are uptodate
    local rc=1
    
    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_amu.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}

        is_drbd_mirror_uptodate $mlab
        if [ $? -eq 0 ]; then
        	debug "Not all mirrors are up to date"
        	rm -f $tmpf  >/dev/null        	
        	return 0
        fi
                 
    done < $tmpf
        
    rm -f $tmpf  >/dev/null
    return $rc

}

is_drbd_mirror_uptodate () {
	local mirror=$1
	local rc=0
	
    local mirror_state=`drbd-overview | grep $mirror | awk '{print $4}'`
    debug "Mirror state for $mirror is:$mirror_state"
    case "$mirror_state" in         
        UpToDate/UpToDate)            
            rc=1
            ;;
        *)
            rc=0
            ;;
    esac
    return $rc
	
}

confirm_all_in_sync () {
    local timeout_in_seconds=${1}
    local wait_forever=0
    local number_of_tries=1

    if [ -z "$timeout_in_seconds" ]; then
        timeout_in_seconds=-1
    fi

    if [ $timeout_in_seconds -eq 0 ]; then
        wait_forever=1
        debug "confirm_all_in_sync, $name $addr about to loop until we find the meps"
    elif [ $timeout_in_seconds -gt 0 ]; then
        number_of_tries=`expr $timeout_in_seconds / 10`
        number_of_tries=`expr $number_of_tries + 1`
        debug "confirm_all_in_sync, $name $addr about to loop for ${timeout_in_seconds} seconds"
    fi

	local i
    for (( i = 0; i < ${number_of_tries}; i++ ))
    do
        all_mirrors_uptodate
		if [ $? -ne 0 ]; then
			# Just return if they disks are up to date
			return
		fi
		
		# sleep for a while so that the disks can sync some more
		if [ `expr $i + 1` -lt $number_of_tries ]; then
		    debug "Manual failover: Disks not in sync.  Sleeping"
		    sleep 10
        fi
    done
    
    #if we get to this code, it means the data is still not in sync
    fatal_error_rc $FAILOVER_FAILED_DATA_NOT_SYNCD "`eval_gettext "A fail over attempt could not be performed because the data is not synchronized."`"
}

process_move () {
    # See if it's already in a move
    in_move
    if [ $? -eq 1 ]; then
        fatal_error_rc $ALREADY_IN_FAILOVER_ATTEMPT "`eval_gettext "A fail over attempt is already in progress. Use -c to cancel."`"
    fi
    
    confirm_all_in_sync 100
    
    # Move the resource groups to the other node
    local move_out=`LC_ALL=C rgreq -o move $STORAGE_RG 2>&1`
    debug "failover -move cmd:$? output:\n $move_out"
}

#####################################################
# Process the failover parameter
#####################################################
proc_failover () {

    chk_ha_setup
    
    # Make sure we're in dual node mode
    is_single_node_mode
    if [ $? -eq 1 ]; then
    	set_dual_node_mode
    fi

    # See if automation is off and if it is, just turn it on.
    is_automation_on
    local rc=$?
    case $rc in
        0 )
            turn_on_automation
        ;;
        1 ) # Automation is on, check if we're excluded.
            is_excluded_list_empty
            if [ $? -eq 0 ]; then
            	debug "Some nodes are excluded."
            	include_all_nodes
            else
                process_move
            fi
        ;;
        *) # Bad return code
        debug "is_automation_on returned: $rc"
        fatal_error_rc $UNKNOWN_ERROR "`eval_gettext "An unknown error occurred."`"
        ;;
        esac    

    log_command_success
    exit 0
		
}

#####################################################
# Process the cancel parameter
#####################################################
proc_cancel () {
    
    if [[ "$cancel" == "Y" ]]; then
    	
    	chk_ha_setup
    	
        # See if it's already in a move
        in_move
        if [ $? -eq 0 ]; then
            fatal_error "`eval_gettext "A fail over is not in progress."`"
        fi
        # Cancel the Move
        local move_out=`LC_ALL=C rgreq -o movecancel $STORAGE_RG`
    	debug "failover -movecancel cmd:$? output:\n $move_out"
    	
    	log_command_success
        exit 0
    fi

}


#####################################################
# Process the validate parameter
#####################################################
proc_validate () {
    
    if [[ "$validate" == "Y" ]]; then
    	
    	chk_ha_setup
    	
    	# Check to see if there are any reasons we believe the fail over will not be successful
        # Perform any tests here
        chk_nodes_opstate

        if [[ "$immediate" == "Y" ]]; then
            confirm_all_in_sync 1
        else
            confirm_all_in_sync 100
        fi
        
        # If we make it here, everything must be okay
        sm_msg "`eval_gettext "No validation errors found."`"
        
        log_command_success
        exit 0
    fi

}


# Reset any resources that need it
reset_rsrcs () {
	local name
	local opstate
	local nodelist
	local resource
	local reset_rcrs=N
	
	local tmpf=`mktemp /tmp/smha_rr.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
	LC_ALL=C lsrsrc IBM.Application Name OpState NodeNameList ResourceType > $tmpf
	LC_ALL=C lsrsrc IBM.ServiceIP Name OpState NodeNameList ResourceType >> $tmpf
	while read -r line
	do
		case "$line" in
			Name*)
			     name=`echo $line | cut -d' ' -f 3`
			     #debug "name: $name"   
            ;;
			OpState*)
			     opstate=`echo $line | cut -d' ' -f3`
			     #debug "opstate: $opstate"
            ;;
			NodeNameList*)
			     nodelist=`echo $line | cut -d' ' -f3`
			     #debug "nodelist: $nodelist"			     
			;;
			ResourceType*)
		         resource=`echo $line | cut -d' ' -f3`
                 # Resource is the last attribute in the groupings so process the group:
                 # Only process fixed resources (float have fixed too!)
                 if [ "$resource" == "0" ]; then
                    debug "possible reset: $name $opstate $nodelist"
                 	case $opstate in
                 		# 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline
                 		3|4)                 		     
                 		     # Get the node name from the node list form. Delete everything up to the first "
                             node=${nodelist#*\"}
                             node=${node%\"*}
                             node=`echo $node | cut -d'.' -f1`
                 		     is_node_online $node
                 		     if [ $? -eq 1 ]; then 
                   		         debug "Reseting ==> Node: $nodelist Opstate: $opstate : $name"
                 		         if [[ "$name" == \"alias* ]]; then
                 		         	 debug "Reseting IBM.ServiceIP name is: $name"
                         		     reset_out=`LC_ALL=C resetrsrc -s "Name=$name" IBM.ServiceIP 2>&1`
                         		     debug "Reset rc:$? out:$reset_out"
                      		     else
                      		         debug "Reseting IBM.Application $name on nodelist $nodelist"
                         		     reset_out=`LC_ALL=C resetrsrc -s "Name=$name && NodeNameList=$nodelist" IBM.Application 2>&1`
                         		     debug "Reset rc:$? out:$reset_out"
                   		         fi
                                 reset_rcrs=Y
                 		     fi       		     
                 		;;
#                 		*)
#                 		     debug "OpState is: $opstate"
#                 		;;
                 	esac
                 fi
			;;
		esac
	done < $tmpf	
	rm $tmpf
	
	# If we didn't find any resources that were failed, try taking director offline and back online, if it isn't online
	if [ "$reset_rcrs" == "N" ]; then

        # First, make sure the Director resource isn't already online (ie working)		
		is_director_online
	    if [ $? -eq 0 ]; then
	    	
			# Second, make sure that Director's RG isn't already offline, if that's the case then
			# we don't want to create the side effect of bring it online 
			get_rg_status_num $DIRECTOR_RG
			if [ $? -ne 2 ]; then 
			
			  take_rg_offline_n_wait $DIRECTOR_RG
			
			  ch_rg_online $DIRECTOR_RG
			fi
		fi
		
	fi
		
}

reset_tsa () {
	
    local pid=`ps aux | grep IBM.RecoveryR | grep rsct | awk '{print $2}'`

    if [ -n "$pid" ]; then
        kill $pid
    else
        debug "IBM.RecoveryRMd process not found."
    fi
}

reset_meps () {

    # need $HOST and $node defined.	
    if [ -z "$node" ]; then
        get_value_from_cfg node
        if [ -n "$CFG_RETURN_VALUE" ]; then
            # We now have the node name
            node=$CFG_RETURN_VALUE
        else
            fatal_error "Could not retrieve the node value from configuration."
        fi
    fi
    
    $SMHA_DIR/bin/tsa/smhadirstatus.sh 1>/dev/null 2>&1
    if [ $? -eq 0 ]; then
	    register_meps
    else
        debug "Cannot reregister meps...director is not active."
    fi
	
	
}

#####################################################
# Process the reset parameter
#####################################################
proc_reset () {
    
    if [[ "$reset" == "Y" ]]; then
    	
    	chk_ha_setup

        # delete any tsa blocking files that we find    	    	
        rm -f /var/ct/samtb_net_blockreserve_*    	    	
    	    	
        if [[ "$tsa" == "Y" ]]; then
        	reset_tsa
        elif [[ "$meps" == "Y" ]]; then
            reset_meps
        else
            reset_rsrcs        
        fi
    	
    	sm_msg "`eval_gettext "Director HA has been reset."`"
    	log_command_success
        exit 0
    fi

}


###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    echo_help
    exit 0
fi

ORIG_PARMS="$*"
LOGCOMMAND=ON

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help
# If start is specified
proc_start
# If stop is specified
proc_stop
# If suspend is specified
proc_suspend
# If resume is specified
proc_resume
# If reset is specified
proc_reset
# If the validate parameter is specified with the failover parm
proc_validate
# If the cancel parameter is specified with the failover parm
proc_cancel
# If failover is specified
proc_failover

log_command_success

exit 0
