#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help t terminate"
PARM_MULTI="network-alias network-alias6"
PARM_TRANSLATION="n node \
f parameter-file \
p password \
L local-replication-address \
R remote-replication-address \
i network-alias \
I network-alias6 \
q network-quorum \
g agent-manager-address \
t terminate \
D dhcp \
N ntp \
B dns \
h help" 
PARM_SINGLES="help"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh

INIT_CMD=$CMD_DIR/smhainit.sh

###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    # list_parm_vars
      
    for option in $CMD_PARMS
    do
#       echo "option: $option"
        # For each option, make sure the minimum parameters are specified
## !!!! Checking is not complete....just enough to check the code is done       
        case $option in 
            node)
               ensure_parms "node password"
            ;;
            *)
                # nothing to do for all other parameters
                echo " " >> /dev/null
            ;;
        esac
    done

        
}

echo_help () {
        echo "${PROGNAME}: [-h][-n <node>]"
        echo ""
        echo "The ${PROGNAME} command is used to start the HA daemon."
        echo "Options:"
        echo ""
        echo "  -h,--help                       Display this help"
        echo "  -n,--node                       Specifies the companion node in the configuration"
        echo " "
}

#####################################################
# Process the help parameter
#####################################################
proc_help () {
    
    if [[ "$help" == "Y" ]]; then
        echo_help
        exit 0
    fi

}

#####################################################
# Process the node parameter
#####################################################
proc_node () {
    
    # if the node parameter was specified then handle it.
    if [ -n "$node" ]; then
    	        

        does_ha_config_exist
        if [ $? -eq 1 ]; then
        	return_error 1 "HA Configuration exists."
        fi
                                
    	######################
    	# setup both nodes
        ######################
        $INIT_CMD -n $node -p $password
        RC=$?
        if [ $RC -ne 0 ]; then
        	return_error $RC "Error configuring nodes. RC=$RC"
        fi
        
        ######################
        # sync the firewall
        ######################
        $INIT_CMD -f
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the firewall. RC=$RC"
        fi
        
        ######################
        # sync the DNS resolver
        ######################
        $INIT_CMD -B
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the DNS resolver. RC=$RC"
        fi
        
        ######################
        # sync the NTP 
        ######################
        $INIT_CMD -N
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the Network Time Protocol Daemon. RC=$RC"
        fi
        
        
        ######################
        # sync the authentication mechanisms (kerberos/ldap)
        ######################
        $INIT_CMD -a
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing remote authentication configuration. RC=$RC"
        fi
        
        ######################
        # sync the users
        ######################
        $INIT_CMD -u
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing users. RC=$RC"
        fi

        ######################
        # setup drbd
        ######################
        
	    # Handle defaulting the local and remote addresses
	    local loc_addr
	    local rmt_addr
	    # if specified, then use it, otherwise default it   
	    if [ -n "$local_replication_address" ]; then
	        loc_addr=$local_replication_address
	    else
	        loc_addr=`gethostip -d $HOST`
	    fi
	
	    # if specified, then use it, otherwise default it
	    if [ -n "$remote_replication_address" ]; then
	        rmt_addr=$remote_replication_address
	    else
	        rmt_addr=`gethostip -d $node`
	    fi
        
        $INIT_CMD -d -L $loc_addr -R $rmt_addr        
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error setting up mirroring. RC=$RC"
        fi
        
        ######################
        # prep the nodes
        ######################
        $INIT_CMD --prep
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error preparing nodes. RC=$RC"
        fi
                             
        ######################
        # create the tsa model
        # and create the conf
        ######################
        #echo "Running: $INIT_CMD --tsa -n $node"

        # Reconstruct the parameter to send to INIT

        ORIG_PARMS=$CMD_PARMS
        EXCLUDE_PARMS="parameter_file"
        FLAG_PARMS=$PARM_FLAGS
        construct_parms_string
        #   output: PARM_STRING

        $INIT_CMD "--tsa $PARM_STRING"

        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error creating TSA model. RC=$RC"
        fi
       
       exit 0
    fi
    
}

#####################################################
# Process the terminate parameter
#####################################################
proc_terminate () {
	
    # if the terminate parameter was specified then handle it.
    if [ -n "$terminate" ]; then
        $INIT_CMD -t
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error terminating configuration. RC=$RC"
        fi
                
        exit 0
    fi
	
}

#####################################################
# Process the dhcp parameter
#####################################################
proc_dhcp () {
	
    # if the dhcp parameter was specified then handle it.
    if [ -n "$dhcp" ]; then
        $INIT_CMD -D $dhcp
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error configuring dhcp. RC=$RC"
        fi
    fi
	
}

###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    echo_help
    exit 0
fi

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help
# If node is specified then they are doing initial setup
proc_node
# If terminate is specified
proc_terminate
# if dhcp is specified
proc_dhcp

exit 0
