#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
EXTPROGNAME="smhastatus"

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh

PARM_FLAGS="h help d detail r refresh m mirrors"
PARM_MULTI=""
PARM_TRANSLATION="r refresh \
d detail \
m mirrors \
h help"
PARM_SINGLES="help"
PARM_VALID=$PARM_TRANSLATION

LOOP_INTERVAL=20

###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    #list_parm_vars 
    
    if [ -n "$mirrors" ]; then
        ensure_parms_not "mirrors" "detail"
    fi
     
    
    
}

echo_help () {
	echo -e "`eval_gettext \"\\\${EXTPROGNAME}: [-h|--help][-r|--refresh][-d|--detail][-m|--mirrors]\"`"
    echo -e "`eval_gettext \"The \\\${EXTPROGNAME} command displays the status of the Director HA environment.\"`"
    echo -e "`eval_gettext "Options:\n"`"
    echo -e "`eval_gettext "\t-h,--help\tDisplay this help"`"
    echo -e "`eval_gettext "\t-d,--detail\tShow detailed status"`"
    echo -e "`eval_gettext "\t-m,--mirrors\tShow mirroring status"`"
    echo -e "`eval_gettext "\t-r,--refresh\tContinuously display the status if it changes\n"`"
}

#####################################################
# Process the help parameter
#####################################################
proc_help () {
    
    if [[ "$help" == "Y" ]]; then
        echo_help
        exit 0
    fi

}
#####################################################
# Process the lssam status
#####################################################
proc_lssam () {
    local lssam_refresh=
    if [ -n "$refresh" ]; then
        lssam_refresh="-top"
    fi
    
    lssam $lssam_refresh
}


#####################################################
# Process the ha status
#####################################################
proc_ha_status () {
    local RC=0
    local do_refresh=1
    local last_state="DONTKNOW"
    local state
    local newline=1
    local previous_newline
    local sprogress=0
        
    until [ $do_refresh -eq 0 ]; do
        
        # Save the previous newline state and reset it.
        previous_newline=$newline
        newline=1
        
        # First check if HA is configured
        is_configured
        if [ $? -eq 0 ]; then
        	# This node is not fully configured, see if it's in progress
            is_config_in_progress
            local progress=$?
            # Make sure the progress goes up
            if [ $progress -gt $sprogress ]; then
                sprogress=$progress
            fi
            # Don't let the progress get above 95 because we have more to do than DRBD.
            if [ $sprogress -gt 95 ]; then
                sprogress=95
            fi
#            debug "progress:$progress, sprogress:$sprogress"
            if [ $progress -ne 0 ]; then
                # This node is being configured
                newline=0           
                state="`eval_gettext \"\rConfiguration in progress, \\\${sprogress}% complete.\"`"
                # Set RC to six, which means configuring in progress
                RC=6
                
            else
                # This node is not configured           
                state="`eval_gettext "Not Configured"`"
                # Set RC to four, which means Error
                RC=4
            fi
            
        else
            sprogress=0
            # So we think it's setup, but is it terminating?
            is_terminating
            if [ $? -eq 1 ]; then
                # The configuration is being removed
                state="`eval_gettext "Removing Configuration"`"
                # Set RC to 2 which means ending
                RC=2   
            else
                # Check if this node is passive            
                is_passive
                if [ $? -eq 0 ]; then
                 
                    # This is the active node           
                    state="`eval_gettext "Active"`"
                    
                    # Set RC to zero, which means Active
                    RC=0    
                else
                    # This is the passive node
                    state="`eval_gettext "Passive"`"
                    # Set RC to 3 which means inactive
                    RC=3                       
                fi
            fi
        fi
                
        # Check if we should output the state or not
        if [ "$last_state" != "$state" ]; then
            if [ $newline -eq 0 ]; then
                if [ -z "$refresh" ]; then
                	echo -e "$state   "
                else	
	            	# Print the state w/o the \n
	            	echo -en "$state   "
                fi
            else
                if [ $previous_newline -eq 0 ]; then
                	echo -e "\n$state"
                else
                    echo -e "$state"                
                fi
            fi
            
            # Record the last state
            last_state=$state
            
        fi
        
        
        # If the refresh flag is not set then don't loop again
        if [ -z "$refresh" ]; then
            do_refresh=0
        else
            # Sleep before looping
            sleep $LOOP_INTERVAL
        fi
    
    done
    
    exit $RC

}

get_drbd_sync_percent () {
	local RC=0
	
	# if drbd is not running, return 0
	if [ ! -e /proc/drbd ]; then
	    return 0
	fi
	
    # (43)    sync_progress = (integer) 417     [len: 4]
    line=`/sbin/drbdadm state all 2>/dev/null | grep integer`
    if [ -n "$line" ]; then
        
        #    l2=`echo ${line##*(integer)} | sed 's/[0-9][0-9][0-9]/&#/g'`
        #debug "l2: $l2"    
        #    l3=${l2%#*}
        #debug "l3: $l3"

        l3=`echo ${line##*(integer)} | awk '{print $1}'`
        let "l4 = $l3 / 10"
        debug "drbd sync percent returned:$l4:"
        if [ $l4 -gt 2 ]; then
            RC=$l4
        fi
    else
        local uptodate=`/etc/init.d/drbd status | grep UpToDate/UpToDate`
        if [ -n "$uptodate" ]; then
        	RC=95
        fi
    fi
    return $RC
}    

is_config_in_progress () {
    local RC=0
    # First check if the daemon file is present
    if [ -f $DM_CONF ]; then
    	# The node is a least partially setup
    	RC=5
    	get_drbd_sync_percent
    	local pcent=$?
    	if [ $pcent -gt 5 ]; then
    		RC=$pcent
    	fi
    	
    fi

    return $RC            	
}

is_config_in_progress_2 () {
    local RC=0
    STATUS_LINE=
    # First check if the daemon file is present
    if [ -f $DM_CONF -o -f $BROKER_KS ]; then
        # The node is a least partially setup
        RC=5
        if [ -f $CFG_STATUS_FILE ]; then
        	STATUS_LINE=`tail -n 1 $CFG_STATUS_FILE`
        else
            STATUS_LINE="`eval_gettext "Configuration is starting."`"
        fi        
        
    fi

    return $RC              
}


proc_ha_status___new () {
    local RC=0
    local do_refresh=1
    local last_state="DONTKNOW"
    local state
    local newline=1
    local previous_newline
        
    until [ $do_refresh -eq 0 ]; do
        
        # Save the previous newline state and reset it.
        previous_newline=$newline
        newline=1
        
        # First check if HA is configured
        is_configured
        if [ $? -eq 0 ]; then
            # This node is not fully configured, see if it's in progress
            is_config_in_progress_2
            local progress_rc=$?
            local last_status=$STATUS_LINE
            if [ $progress_rc -ne 0 ]; then
                # This node is being configured
                #newline=0           
                state="\r$last_status"
                # Set RC to six, which means configuring in progress
                RC=6
                
            else
                # This node is not configured           
                state="`eval_gettext "Not Configured"`"
                # Set RC to four, which means Error
                RC=4
            fi
            
        else
            # So we think it's setup, but is it terminating?
            is_terminating
            if [ $? -eq 1 ]; then
                # The configuration is being removed
                state="`eval_gettext "Removing Configuration"`"
                # Set RC to 2 which means ending
                RC=2   
            else
                # Check if this node is passive            
                is_passive
                if [ $? -eq 0 ]; then
                 
                    # This is the active node           
                    state="`eval_gettext "Active"`"
                    
                    # Set RC to zero, which means Active
                    RC=0    
                else
                    # This is the passive node
                    state="`eval_gettext "Passive"`"
                    # Set RC to 3 which means inactive
                    RC=3                       
                fi
            fi
        fi
                
        # Check if we should output the state or not
        if [ "$last_state" != "$state" ]; then
            if [ $newline -eq 0 ]; then
                if [ -z "$refresh" ]; then
                    echo -e "$state   "
                else    
                    # Print the state w/o the \n
                    echo -en "$state   "
                fi
            else
                if [ $previous_newline -eq 0 ]; then
                    echo -e "\n$state"
                else
                    echo -e "$state"                
                fi
            fi
            
            # Record the last state
            last_state=$state
            
        fi
        
        
        # If the refresh flag is not set then don't loop again
        if [ -z "$refresh" ]; then
            do_refresh=0
        else
            # Sleep before looping
            sleep $LOOP_INTERVAL
        fi
    
    done
    
    exit $RC

}


#####################################################
# Process the ha mirror status
#####################################################
mirror_status () {
	local RC=0
    local do_refresh=1
        
    until [ $do_refresh -eq 0 ]; do

        if [ -n "$refresh" ]; then
        	clear
        fi

        mirror_header
        mirror_sync_status
 
        # If the refresh flag is not set then don't loop again
        if [ -z "$refresh" ]; then
            do_refresh=0
        else
            # Sleep before looping
            sleep $LOOP_INTERVAL
        fi
    
    done
    
    exit $RC
	
}

mirror_header () {
    echo "`eval_gettext "HA Configuration Mirror Status"`"
}

mirror_sync_status () {
    local tmpf=`mktemp /tmp/smha_mss.XXXXXX`  || fatal_error "Could not make temporary file"
    drbdadm status 2>/dev/null | grep -o -e "name=[A-Za-z0-9\"]*" -e "cs=[A-Za-z0-9\"]*" -e "ds1=[A-Za-z\"]*" -e "ds2=[A-Za-z\"]*" -e "percent=[0-9.\"]*" >> $tmpf
    
    name_read=
    line_outputted=
    
    while read -r line
    do
        case "$line" in
            name*)
                if [ -n "$name_read" ]; then
                    mirror_output_line
                fi
                name=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                name=/$name
                name_read=1
            ;;
            cs*)
                cs=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_cs
            ;;
            ds1*)
                ds1=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_ds $ds1
                ds1=$DSTATE
            ;;
            ds2*)
                ds2=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_ds $ds2
                ds2=$DSTATE
            ;;
            percent*)
                percent=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                mirror_output_line
            ;;

            *)
                debug "line not handled: $line"
            ;;
        esac
    done < $tmpf    
    rm $tmpf
    if [ -n "$name_read" ]; then
        mirror_output_line
    fi
    if [ -z "$line_outputted" ]; then
        echo "`eval_gettext "No status is available."`"
    fi
    
    
}
translate_cs () {
        case "$cs" in
            SyncTarget)
                cs="`eval_gettext "Synchronizing from other node"`"
            ;;
            SyncSource)
                cs="`eval_gettext "Synchronizing to other node"`"
            ;;
            Connected)
                cs="`eval_gettext "Connected"`"
            ;;
            Verif*)
                cs="`eval_gettext "Connected"`"
            ;;
            Paused*)
                cs="`eval_gettext "Paused"`"
            ;;
            WFConnection)
                cs="`eval_gettext "Waiting for connection"`"
            ;;
            StartingSyn*)
                cs="`eval_gettext "Starting synchronization"`"
            ;;
            Unconnected)
                cs="`eval_gettext "Not connected"`"
            ;;
            Timeout)
                cs="`eval_gettext "Not connected"`"
            ;;
            Disconnecting)
                cs="`eval_gettext "Not connected"`"
            ;;
            StandAlone)
                cs="`eval_gettext "Not connected"`"
            ;;
            *)
                debug "cs not recongized: $cs"
                cs="`eval_gettext "Error"`"
                
            ;;
        esac
	
}

translate_ds () {
    local dstate=$1
    	   
    case "$dstate" in
        Inconsistent)
            DSTATE="`eval_gettext "Inconsistent"`"
        ;;
        UpToDate)
            DSTATE="`eval_gettext "Up-to-date"`"
        ;;
        Outdated)
            DSTATE="`eval_gettext "Outdated"`"
        ;;
        Diskless)
            DSTATE="`eval_gettext "Error"`"
        ;;
        Attaching)
            DSTATE="`eval_gettext "Connecting"`"
        ;;
        Consistent)
            DSTATE="`eval_gettext "Consistent"`"
        ;;
        *)
            debug "dstate not recongized: $dstate"
            DSTATE="`eval_gettext "Unknown"`"
            
        ;;
    esac
    
}

mirror_output_line () {

    echo "`eval_gettext \"Name: \\\$name\"`"
    echo "`eval_gettext \"    Connection status: \\\$cs\"`"
    echo "`eval_gettext \"    Status on this node: \\\$ds1\"`"
    echo "`eval_gettext \"    Status on other node: \\\$ds2\"`"
    if [ -n "$percent" ]; then
        echo "`eval_gettext \"    Synchronization percent: \\\$percent\"`"
    fi
    name=
    cs=
    ds1=
    ds2=
    percent=
    name_read=
    line_outputted=1

}


###################################
## Main code                     ##
###################################

trap 'exit 0' SIGINT SIGTERM

proc_parms $*
##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help

if [ -n "$mirrors" ]; then
    mirror_status
fi

if [ -z "$detail" ]; then
    # Process the ha status
    proc_ha_status___new   
else
    proc_lssam
fi


